@@ -15,7 +15,6 @@
 #include "command_lexer.h"
 #include "tgdb.h"
 #include "interface.h"
-#include "error.h"
 #include "tokenizer.h"
 #include "cgdb.h"
 
@@ -28,9 +27,9 @@ extern struct tgdb *tgdb;
  *
  */
 
+extern int auto_source_reload;
 extern int regex_icase ;
 extern int shortcut_option ;
-extern int line_coverage_option;
 extern int highlight_tabstop;
 extern int config_wrapscan;
 
@@ -59,10 +58,10 @@ static struct ConfigVariable
     void *data;
 } VARIABLES[] = {
     // keep this stuff sorted! !sort
+	/* autosourcereload */  { "autosourcereload", "asr", CONFIG_TYPE_BOOL, &auto_source_reload },
     /* escdelay   */ 		{ "escdelay", "escdelay", CONFIG_TYPE_FUNC_INT, command_set_esc_sequence_timeout },
     /* focus      */ 		{ "focus", "fo", CONFIG_TYPE_FUNC_STRING, command_set_focus },
     /* ignorecase */ 		{ "ignorecase", "ic", CONFIG_TYPE_BOOL, &regex_icase },
-    /* line_coverage */ 	{ "line_coverage", "lc", CONFIG_TYPE_BOOL, &line_coverage_option },
     /* shortcut   */ 		{ "shortcut", "sc", CONFIG_TYPE_BOOL, &shortcut_option },
 	/* showtgdbcommands */ 	{ "showtgdbcommands", "stc", CONFIG_TYPE_FUNC_BOOL, &command_set_stc },
     /* syntax */      		{ "syntax", "syn", CONFIG_TYPE_FUNC_STRING, command_set_syntax_type }, 
@@ -85,6 +84,7 @@ static int command_do_quit_force( void );
 static int command_do_run( void );
 static int command_do_step( void );
 static int command_search_next( void );
+static int command_source_reload( void );
 static int command_start_tty( void );
 static int command_toggle_tty( void );
 
@@ -100,6 +100,8 @@ static struct commands
 } COMMANDS[] = {
     // keep this stuff sorted, you can use !sort in vi
     /* continue    */ { "continue",    command_do_continue },
+    /* edit        */ { "e",           command_source_reload },
+    /* edit        */ { "edit",        command_source_reload },
     /* finish      */ { "finish",      command_do_finish },
     /* help        */ { "help",        command_do_help },
     /* insert      */ { "insert",      command_focus_gdb },
@@ -303,6 +305,16 @@ int command_search_next( void )
     return 0;
 }
 
+int command_source_reload( void )
+{
+	struct sviewer *sview = if_get_sview ();
+
+	if ( source_reload ( sview, sview->cur->path, 1 ) == -1 )
+		return -1;
+
+    return 0;
+}
+
 int command_start_tty( void )
 {
     /* FIXME: Find out what this is supposed to do and implement */
