@@ -49,7 +49,7 @@
 /* Local Includes */
 #include "cgdb.h"
 #include "config.h"
-#include "error.h"
+#include "logger.h"
 #include "interface.h"
 #include "kui_term.h"
 #include "scroller.h"
@@ -129,7 +129,6 @@ static int regex_search;                /* Currently searching text ? */
 static int regex_direction;             /* Direction to search */
 /*static*/ int regex_icase = 0;         /* Case insensitive (0), sensitive */
 /*static*/ int shortcut_option = 0;     /* Flag: Shortcut options enabled */
-		   int line_coverage_option = 0;/* If lines should be highlighted when covered */
            int config_wrapscan = 1;     /* Wrap around searching */
 
 static char last_key_pressed = 0;       /* Last key user entered in cgdb mode */
@@ -672,7 +671,8 @@ static void signal_handler(int signo) {
             write( resize_pipe[1], &c, sizeof(int));
             resize = 1;
         }
-    } else if ( signo == SIGINT || signo == SIGTERM || signo == SIGQUIT )
+    } else if ( signo == SIGINT || signo == SIGTERM || 
+                signo == SIGQUIT || signo == SIGCHLD )
 		tgdb_signal_notification ( tgdb, signo );
 }
 
@@ -766,6 +766,7 @@ static void if_run_command(struct sviewer *sview) {
 static int capture_regex(struct sviewer *sview) {
    int c;
    
+   int orig_line = sview->cur->sel_line;
    /* Initialize the function for finding a regex and tell user */
    regex_search = 1;
    regex_line_pos = 0;
@@ -782,6 +783,7 @@ static int capture_regex(struct sviewer *sview) {
          *regex_line = '\0';
          regex_search = 0;
          source_search_regex(sview, regex_line, 2, regex_direction, regex_icase);
+         sview->cur->sel_line = orig_line;
          if_draw();
          return 1;
       }
@@ -935,12 +937,6 @@ static void source_input(struct sviewer *sview, int key)
         case 'G': /* end of file */
             source_set_sel_line(sview, 10000000);
             break;
-        case 'f':
-            source_next(sview);
-            break;
-        case 'b':
-            source_previous(sview);
-            break;
         case '=':
            // inc window by 1
            increase_win_height( 0 );
@@ -1024,22 +1020,27 @@ static int set_up_signal(void) {
 	action.sa_flags = 0;
 
 	if(sigaction(SIGWINCH, &action, NULL) < 0) {
-	  err_ret("%s:%d -> sigaction failed ", __FILE__, __LINE__);
+	  logger_write_pos ( logger, __FILE__, __LINE__, "sigaction failed ");
 	  return -1;
 	}
 
     if(sigaction(SIGINT, &action, NULL) < 0) {
-        err_ret("%s:%d -> sigaction failed ", __FILE__, __LINE__);
+        logger_write_pos ( logger, __FILE__, __LINE__, "sigaction failed ");
 		return -1;
 	}	
 
     if(sigaction(SIGTERM, &action, NULL) < 0) {
-        err_ret("%s:%d -> sigaction failed ", __FILE__, __LINE__);
+        logger_write_pos ( logger, __FILE__, __LINE__, "sigaction failed ");
 		return -1;
 	}
 
     if(sigaction(SIGQUIT, &action, NULL) < 0) {
-        err_ret("%s:%d -> sigaction failed ", __FILE__, __LINE__);
+        logger_write_pos ( logger, __FILE__, __LINE__, "sigaction failed ");
+		return -1;
+	}
+
+    if(sigaction(SIGCHLD, &action, NULL) < 0) {
+        logger_write_pos ( logger, __FILE__, __LINE__, "sigaction failed ");
 		return -1;
 	}
 
