@@ -52,6 +52,7 @@
 #include "sys_util.h"
 
 int sources_syntax_on = 1;
+int auto_source_reload = 0;
 
 /* --------------- */
 /* Local Functions */
@@ -72,95 +73,6 @@ static int verify_file_exists(const char *path) {
     return 1;
 }
 
-/* source_queue_init: Initializes the queue to be empty.
- * ------------------
- *
- *   sview:  The source viewer object
- */
-void source_queue_init(struct sviewer *sview) {
-    int i;
-    
-    for ( i = 0; i < QUEUE_SIZE; i++ )
-        sview->queue[i] = NULL;
-    sview->queue_index = 0;
-    sview->queue_top_index = 0;
-}
-
-/* source_queue_remove: Removes a specific file from the queue
- * --------------------
- *
- *   sview:  The source viewer object
- */
-void source_queue_remove(struct sviewer *sview, struct list_node *node) {
-    int i, j;
-    
-    for ( i = 0; i < sview -> queue_top_index; i++ ) {
-        if ( sview->queue[i] == node ) {    /* Found */
-            sview->queue[i] = NULL;
-            for ( j = i; j < sview -> queue_top_index; j++)
-               sview->queue[j] = sview->queue[j + 1]; 
-
-            for ( j = sview ->queue_top_index; j < QUEUE_SIZE - 1; j++)
-               sview->queue[j] = NULL; 
-            sview->queue_top_index--;
-            break;
-        }
-    }
-}
-
-/* source_queue_push: Removes a specific file from the queue
- * --------------------
- *
- *   sview:  The source viewer object
- */
-void source_queue_push(struct sviewer *sview, struct list_node *node) {
-    int i = sview -> queue_top_index, j, k;
-
-    /* If the file is already in the queue, just move it to the top */
-    for ( j = 0; j < i; j++ ) {
-        /* Found the file, swap the rest down and put it on top */
-        if ( sview->queue[j] == node ) {
-            for ( k = j; k < i; k++ )
-               sview->queue[k] = sview->queue[k + 1]; 
-
-            sview->queue[i - 1] = node;
-            return;
-        }
-    }
-
-    /* Add to the non-full queue */
-    if ( i < QUEUE_SIZE ) {
-        sview->queue[i] = node;    
-        sview->queue_index = i;
-        sview->queue_top_index++;
-        for ( j = i + 1; j < QUEUE_SIZE; j++)
-            sview->queue[j] = NULL;
-    } else if ( i == QUEUE_SIZE - 1) { 
-        for ( j = 1; j < QUEUE_SIZE - 1; j++)
-            sview->queue[j - 1] = sview->queue[j];
-        sview->queue[j] = node;
-    }
-}
-
-void source_previous(struct sviewer *sview) {
-    /* Only one file, don't switch */
-    if ( sview->queue_index < 1 )
-        return;
-
-    sview->queue_index--;
-    sview->cur = sview->queue[sview->queue_index];
-}
-
-void source_next(struct sviewer *sview) {
-    /* Check if we are at the last file viewed */
-    if ( sview->queue_index == QUEUE_SIZE - 1 || 
-         sview->queue[sview->queue_index + 1] == NULL )
-        return;
-
-    sview->queue_index++;
-    sview->cur = sview->queue[sview->queue_index];
-}
-
 /* get_relative_node:  Returns a pointer to the node that matches the 
  * ------------------  given relative path.
  
@@ -173,6 +85,7 @@ static struct list_node *get_relative_node(struct sviewer *sview, const char *lp
     struct list_node *cur;
     
     for (cur = sview->list_head; cur != NULL; cur = cur->next)
+      if ( cur->lpath ) 
          if (strcmp(lpath, cur->lpath) == 0)
              return cur;
     
@@ -191,12 +104,93 @@ static struct list_node *get_node(struct sviewer *sview, const char *path)
     struct list_node *cur;
     
     for (cur = sview->list_head; cur != NULL; cur = cur->next)
+      if ( cur->path )
          if (strcmp(path, cur->path) == 0)
              return cur;
     
     return NULL; 
 }
 
+/**
+ * Get's the timestamp of a particular file.
+ *
+ * \param path
+ * The path to the file to get the timestamp of
+ *
+ * \param timestamp
+ * The timestamp of the file, or 0 on error.
+ * 
+ * \return
+ * 0 on success, -1 on error.
+ */
+static int get_timestamp ( const char *path, time_t *timestamp ) {
+	struct stat s;
+	int val;
+
+	if ( !path )
+		return -1;
+
+	if ( !timestamp )
+		return -1;
+
+	*timestamp = 0;
+
+	val = stat ( path, &s );
+
+	if ( val ) /* Error on timestamp */
+		return -1;
+
+	*timestamp = s.st_mtime;
+
+	return 0;
+}
+
+static int release_file_buffer ( struct buffer *buf ) {
+	int i;
+
+	/* Nothing to free */
+	if ( !buf ) 
+		return 0;
+
+	for ( i = 0; i < buf->length; ++i ) {
+		free ( buf->tlines[i] );
+		buf->tlines[i] = NULL;
+	}
+
+	free ( buf->tlines );
+	buf->tlines = NULL;
+	buf->length = 0;
+	buf->cur_line = NULL;	
+	buf->max_width = 0;
+    free ( buf->breakpts );
+	buf->breakpts = NULL;
+
+	return 0; 
+}
+
+/** 
+ * Remove's the memory related to a file.
+ *
+ * \param node
+ * The node who's file buffer data needs to be freed.
+ *
+ * \return
+ * 0 on success, or -1 on error.
+ */
+static int release_file_memory ( struct list_node *node ) {
+	if ( !node )
+		return -1;
+	
+	/* Free the buffer */
+	if ( release_file_buffer ( &node->buf ) == -1 )
+		return -1;
+
+	if ( release_file_buffer ( &node->orig_buf ) == -1 )
+		return -1;
+
+	return 0;
+}
+
 /* load_file:  Loads the file in the list_node into its memory buffer.
  * ----------
  *
@@ -222,17 +216,26 @@ static int load_file(struct list_node *node)
 	 */
 	node->orig_buf.length = 0;
 	node->orig_buf.tlines = NULL;
+    node->orig_buf.breakpts  = NULL;
+    node->orig_buf.cur_line  = NULL;
 	node->orig_buf.max_width = 0;
 
+	/* Stat the file to get the timestamp */
+	if ( get_timestamp ( node->path, &(node->last_modification) ) == -1 )
+		return 2;
+
     if (!(file = fopen(node->path, "r")))
         return 1;
 
 	while (!feof(file)){
 		if (fgets(line, MAX_LINE, file)){
-			if (line[strlen(line)-1] == '\n')
-				line[strlen(line)-1] = 0;
-			if (line[strlen(line)-1] == '\r')
-				line[strlen(line)-1] = 0;
+            int length = strlen ( line );
+            if ( length > 0 ) {
+                if (line[length-1] == '\n')
+                    line[length-1] = 0;
+                if (line[length-1] == '\r')
+                    line[length-1] = 0;
+            }
 			if (strlen(line) > node->orig_buf.max_width)
 				node->orig_buf.max_width = strlen(line);
 
@@ -267,11 +270,6 @@ static int load_file(struct list_node *node)
     for (i = 0; i < node->buf.length; i++)
        node->buf.breakpts[i] = 0;
 
-	/* Initialize the coverage */
-	node->covered_lines =  (int *)xmalloc ( sizeof ( int ) * node->buf.length);
-	for ( i = 0; i < node->buf.length; i++ )
-		node->covered_lines[i] = 0;
-
     return 0;
 }
 
@@ -294,8 +292,6 @@ struct sviewer *source_new(int pos_r, int pos_c, int height, int width)
     rv->cur       = NULL;
     rv->list_head = NULL;
 
-    source_queue_init(rv);
-
     return rv;
 }
 
@@ -307,6 +303,7 @@ int source_add(struct sviewer *sview, const char *path)
     
     new_node = malloc(sizeof(struct list_node));
     new_node->path         = strdup(path);
+    new_node->lpath        = NULL;
     new_node->buf.length   = 0;
     new_node->buf.tlines    = NULL;    /* This signals an empty buffer */
     new_node->sel_line     = 0;
@@ -315,6 +312,7 @@ int source_add(struct sviewer *sview, const char *path)
     new_node->sel_col_rend = 0;
     new_node->sel_rline    = 0;
     new_node->exe_line     = 0;
+	new_node->last_modification = 0; /* No timestamp yet */
 
     if (sview->list_head == NULL){
         /* List is empty, this is the first node */
@@ -365,13 +363,14 @@ int source_del(struct sviewer *sview, const char *path)
     if (cur == NULL)
         return 1;      /* Node not found */
 
-    source_queue_remove(sview, cur);
-
     /* Release file buffer, if one is in memory */   
     if (cur->buf.tlines){
         for (i = 0; i < cur->buf.length; i++) {
             free(cur->buf.tlines[i]);
-            free(cur->buf.cur_line);
+            if ( cur->buf.cur_line ) {
+                free(cur->buf.cur_line);
+                cur->buf.cur_line = NULL;
+            }
             free(cur->orig_buf.tlines[i]);
         }
     }
@@ -421,7 +420,6 @@ int source_display(struct sviewer *sview, int focus)
     int lwidth;
     int line;
     int i;
-	extern int line_coverage_option;
 
     /* Check that a file is loaded */
     if (sview->cur == NULL || sview->cur->buf.tlines == NULL){
@@ -512,32 +510,6 @@ int source_display(struct sviewer *sview, int focus)
 						hl_wprintw(sview->win, sview->cur->orig_buf.tlines[line], width-lwidth-2, sview->cur->sel_col);
 				}
             }
-            /* Look for covered_lines */
-            else if (line_coverage_option && sview->cur->covered_lines[line]){
-                wattron(sview->win, COLOR_PAIR(CGDB_COLOR_MAGENTA));
-                wattron(sview->win, A_BOLD);
-                wprintw(sview->win, fmt, line+1);
-                wattroff(sview->win, COLOR_PAIR(CGDB_COLOR_MAGENTA));
-                if (!focus)
-                    wattroff(sview->win, A_BOLD);
-                waddch(sview->win, VERT_LINE);
-                if (focus)
-                    wattroff(sview->win, A_BOLD);
-                waddch(sview->win, ' ');
-
-				/* I know this is rediculous, it needs to be reimplemented */
-				if ( sources_syntax_on ) {
-					if ( line == sview->cur->sel_line && sview->cur->buf.cur_line != NULL )
-						hl_wprintw(sview->win, sview->cur->buf.cur_line, width-lwidth-2, sview->cur->sel_col);
-					else
-						hl_wprintw(sview->win, sview->cur->buf.tlines[line], width-lwidth-2, sview->cur->sel_col);
-				} else {
-					if ( line == sview->cur->sel_line && sview->cur->buf.cur_line != NULL )
-						hl_wprintw(sview->win, sview->cur->orig_buf.cur_line, width-lwidth-2, sview->cur->sel_col);
-					else
-						hl_wprintw(sview->win, sview->cur->orig_buf.tlines[line], width-lwidth-2, sview->cur->sel_col);
-				}
-            }
             /* Look for breakpoints */
             else if (sview->cur->buf.breakpts[line]){
                 if (sview->cur->buf.breakpts[line] == 1)
@@ -686,9 +658,6 @@ int source_set_exec_line(struct sviewer *sview, const char *path, int line)
     if (!sview->cur->buf.tlines && load_file(sview->cur))
         return 4;
 
-    /* Push file on queue */
-    source_queue_push(sview, sview->cur);
-
     /* Update line, if set */
     if (line--){
         /* Check bounds of line */
@@ -699,9 +668,6 @@ int source_set_exec_line(struct sviewer *sview, const char *path, int line)
         sview->cur->sel_line = sview->cur->exe_line = line;
     }
 
-	/* Add this line to the lines covered */
-	sview->cur->covered_lines[line] = 1;
-
     return 0;
 }
 
@@ -711,8 +677,6 @@ void source_free(struct sviewer *sview)
     while (sview->list_head != NULL)
         source_del(sview, sview->list_head->path);
 
-    source_queue_init(sview);
-
     delwin(sview->win);
 }
 
@@ -735,8 +699,10 @@ int source_search_regex(struct sviewer *sview,
                         int direction, int icase) {
 
     if ( sview == NULL || sview->cur == NULL || regex == NULL ||
-         strlen(regex) == 0 )
+         strlen(regex) == 0 ) {
+        sview->cur->buf.cur_line = NULL;
         return -1;
+    }
 
     return hl_regex(regex, 
                     (const char ** )sview->cur->buf.tlines, 
@@ -788,3 +754,35 @@ void source_clear_breaks(struct sviewer *sview)
         
 }
 
+int source_reload ( struct sviewer *sview, const char *path, int force ) {
+	time_t timestamp;
+    struct list_node *cur;
+    struct list_node *prev = NULL;
+
+	if ( !path )
+		return -1;
+
+	if ( get_timestamp ( path, &timestamp ) == -1 )
+		return -1;
+
+	/* Find the target node */
+	for (cur = sview->list_head; cur != NULL; cur = cur->next){
+		if (strcmp(path, cur->path) == 0)
+			break;
+		prev = cur;
+	}
+	
+	if (cur == NULL)
+		return 1;      /* Node not found */
+
+	if ( ( auto_source_reload || force ) && cur->last_modification < timestamp ) {
+
+		if ( release_file_memory ( sview->cur ) == -1)
+			return -1;
+
+		if ( load_file ( cur ) )
+			return -1;
+	}
+
+	return 0;
+}
