@@ -6,8 +6,12 @@
 #include <string.h>
 #endif /* HAVE_STRING_H */
 
+#if HAVE_STDLIB_H
+#include <stdlib.h>
+#endif /* HAVE_STDLIB_H */
+
 #include "annotate.h"
-#include "error.h"
+#include "logger.h"
 #include "data.h"
 #include "commands.h"
 #include "globals.h"
@@ -30,7 +34,7 @@ static int handle_misc_pre_prompt(struct annotate_two *a2, const char *buf, size
    /* If tgdb is sending a command, then continue past it */
    if(data_get_state(a2->data) == INTERNAL_COMMAND){
       if(io_write_byte(a2->debugger_stdin, '\n') == -1)
-         err_msg("%s:%d -> Could not send command", __FILE__, __LINE__);
+         logger_write_pos ( logger, __FILE__, __LINE__, "Could not send command");
    } else {
       data_set_state(a2, AT_PROMPT);
    }
@@ -178,6 +182,17 @@ static int handle_display_value(struct annotate_two *a2, const char *buf, size_t
    return 0;
 }
 
+static int handle_exited(struct annotate_two *a2, const char *buf, size_t n, struct tgdb_list *list){
+	char *tmp = (char*)malloc ( sizeof ( char )*(n+1));
+	int *i = (int *)malloc ( sizeof ( int ) );
+	sprintf ( tmp, buf + 7 ); /* Skip the 'exited ' part */
+	*i = atoi ( tmp ) ;
+	free ( tmp );
+	tmp = NULL;
+    tgdb_types_append_command(list, TGDB_INFERIOR_EXITED, i );
+   return 0;
+}
+
 static int NOT_SUPPORTED(struct annotate_two *a2, const char *buf, size_t n, struct tgdb_list *list){
    return 0;
 }
@@ -423,6 +438,11 @@ static struct annotation {
      13,
      handle_display_value
   },
+  {
+	  "exited",
+	  6,
+	  handle_exited
+  },
   {
       NULL,
       (int)NULL,
@@ -436,7 +456,7 @@ int tgdb_parse_annotation(struct annotate_two *a2, char *data, size_t size, stru
       if(strncmp(data, annotations[i].data, annotations[i].size) == 0){
          if(annotations[i].f){
             if(annotations[i].f(a2, data, size, list) == -1){
-               err_msg("%s:%d -> parsing annotation failed\n", __FILE__, __LINE__);
+               logger_write_pos ( logger, __FILE__, __LINE__, "parsing annotation failed");
             } else 
                break; /* only match one annotation */
          }
